import HighlightText from '@sanar/react-native-highlight-text';
import React, {
  forwardRef,
  useEffect,
  useImperativeHandle,
  useRef,
  useState,
} from 'react';
import { FlatList } from 'react-native-gesture-handler';

function ScrollableHighlightedText({ text = '', textToHighlight }, ref) {
  // const [textToHighlight, setTextToSearch] = useState();
  const flatList = useRef();
  const [arrayOfLines, setArrayOfLines] = useState([]);
  const arrayOfIndexes = useRef([]);
  const counter = useRef(-1);

  useImperativeHandle(ref, () => ({
    // searchText(word) {
    //   setTextToSearch(word);
    // },
    scrollToNext() {
      if (arrayOfIndexes.current.length === 0) {
        return;
      }
      try {
        counter.current++;

        flatList.current?.scrollToIndex({
          index: arrayOfIndexes?.current?.[counter.current],
          animated: true,
        });
      } catch (error) {
        counter.current = 0;
        flatList.current?.scrollToIndex({
          index: arrayOfIndexes?.current?.[counter.current],
          animated: true,
        });
      }
    },
    scrollToPrevious() {
      if (arrayOfIndexes.current.length === 0) {
        return;
      }
      try {
        counter.current--;

        flatList.current?.scrollToIndex({
          index: arrayOfIndexes?.current?.[counter.current],
          animated: true,
        });
      } catch (error) {
        counter.current = 0;
        flatList.current?.scrollToIndex({
          index: arrayOfIndexes?.current?.[counter.current],
          animated: true,
        });
      }
    },
  }));

  useEffect(() => {
    setArrayOfLines(text.split(/\r?\n/));
  }, [text]);

  useEffect(() => {
    arrayOfIndexes.current = [];
    for (let index = 0; index < arrayOfLines.length; index++) {
      if (arrayOfLines[index]?.includes(textToHighlight)) {
        arrayOfIndexes.current = [...arrayOfIndexes.current, index];
      }
    }
  }, [arrayOfLines, textToHighlight]);

  return (
    <FlatList
      ref={flatList}
      initialScrollIndex={0}
      data={arrayOfLines}
      initialNumToRender={10000}
      renderItem={({ item, index }) => {
        return (
          <HighlightText
            // eslint-disable-next-line react-native/no-inline-styles
            highlightStyle={{
              backgroundColor: 'yellow',
            }}
            searchWords={[textToHighlight]}
            textToHighlight={item}
          />
        );
      }}
    />
  );
}

export default forwardRef(ScrollableHighlightedText);

// interface HighlightTextProps extends FindAllArgs, TextProps {
//   highlightStyle?: TextStyle;
//   highlightComponent?: React.ComponentType<TextProps>;
//   textComponent?: React.ComponentType<TextProps>;
// }
